#!/usr/bin/env python
# -*- coding: utf-8 -*-
# colorchart_generic.py

"""
# ABOUT
# This script is designed to create color swatches
# according to certain specifications, on A4 paper, with swatches 
# measuring 22x12mm.
# Color patches will be placed in a 7x7 array, colors created to
# match the L, a, and b values for each color. A label is placed under
# each color patch with its name, and the L, a, and b values.
# Each line of the clf file source should contain a color name, then its 
# L, a, and b values separated by tabs. If the Lab data uses commas 
# for the decimal separator, they will be converted to decimal points.
# A dialog asks for a heading for the pages.
# This version 2017/12/29
# © Gregory Pittman and freieFarbe e.V., 2017
"""

import scribus
import re, os, sys, datetime

def SelectAllText(textframe):
    texlen = scribus.getTextLength(textframe)
    scribus.selectText(0,texlen,textframe)
    return

colors = []
L = []
a = []
b = []
index = 0

ypos = -5
scribus.messageBox("About",'This script will take the Lab colour data \nfrom a clf colour file, create those colours\nin Scribus, and in addition create a \ndocument displaying each of these colours.\n\n© Gregory Pittman and freieFarbe e.V., 2017')
colorfile = scribus.fileDialog('Select a color file', 'CLF files(*.clf)')
headingparts = os.path.split(colorfile)
heading = headingparts[-1]
heading = heading[:-4]
header = scribus.valueDialog("Title",'Enter a title for your color chart',heading)
for line in open(colorfile).xreadlines():
    content = line.split('\t')
    fields = len(content)
    if (fields == 4):
        colors.append(content[0])
        Lval = content[fields - 3]
        Lval = re.sub(r',','.',Lval)
        Lval = float(Lval)
        L.append(Lval)
        aval = content[fields - 2]
        aval = re.sub(r',','.',aval)
        aval = float(aval)
        a.append(aval)
        bval = content[fields - 1]
        bval = re.sub(r',','.',bval)
        bval = float(bval)
        b.append(bval)
size = len(colors)
docsize = int(size/49)
plural = 's'
if ((size % 49) > 0):
    docsize = docsize + 1
if docsize == 1:
    plural = ''
if size > 500:
    response = scribus.messageBox('Query', 'You are about to create '+str(size) + ' colors,\n and a document with '+str(docsize)+ ' page' + plural + '.\nThis may take several minutes.\nDo you want to do this?', icon=scribus.ICON_WARNING, button1=scribus.BUTTON_YES, button2=scribus.BUTTON_NO)
else:
    response = scribus.messageBox('Query', 'You are about to create '+str(size) + ' colors,\n and a document with '+str(docsize)+ ' page' + plural + '.\nDo you want to do this?', icon=scribus.ICON_NONE, button1=scribus.BUTTON_YES, button2=scribus.BUTTON_NO)
if response != 16384:
    sys.exit(-1)
#start = datetime.datetime.now().replace(microsecond=0)
scribus.progressReset()
scribus.progressTotal(docsize)
currpage = 1
if scribus.newDoc(scribus.PAPER_A4, (0,0,0,0),scribus.PORTRAIT, 1, scribus.UNIT_POINTS, scribus.NOFACINGPAGES, scribus.FIRSTPAGERIGHT):
    scribus.setUnit(scribus.UNIT_MILLIMETERS)
    scribus.setRedraw(False)
    scribus.createCharStyle("titlechar", "DejaVu Sans Condensed Bold", 18)
    scribus.createParagraphStyle("titlestyle",1,0,0,0,0,0,0,0,0,0,0,"titlechar")
    scribus.createCharStyle("keychar", "DejaVu Sans Condensed Bold", 7.0,'','Black',1.0,'',0,0,0,0,0,0,0,0,0,1,1,-50)
    scribus.createParagraphStyle("keystyle",0,7.0,1,0,0,0,0,0,0,0,0,"keychar")
    scribus.createCharStyle("keyLabchar", "DejaVu Sans Book", 6.0)
    scribus.createParagraphStyle("keyLabstyle",0,7.0,1,0,0,0,0,0,0,0,0,"keyLabchar")
    scribus.createCharStyle("footerchar", "DejaVu Sans Book", 7)
    scribus.createParagraphStyle("footerstyle",0,12.0,2,0,0,0,0,0,0,0,0,"footerchar")
    scribus.createMasterPage('attribution')
    scribus.editMasterPage('attribution')
    footer = scribus.createText(97,285.3,99,5.2)
    scribus.setText('Scribus script created by Gregory Pittman & freieFarbe.de 2017', footer)
    scribus.setStyle("footerstyle", footer)
    leftfooter = scribus.createText(15.5, 281, 80, 9.5)
    scribus.setText(heading + '\nNo. of colours: ' + str(size) + ' ' + u'\u00b7' + ' Page \x1e of \x17', leftfooter)
    SelectAllText(leftfooter)
    scribus.setStyle("footerstyle", leftfooter)
    scribus.setTextAlignment(scribus.ALIGN_LEFT, leftfooter)
    scribus.closeMasterPage()
    scribus.gotoPage(1)

    titleframe = scribus.createText(30, 15, 90, 15)
    scribus.setText(heading, titleframe)
    scribus.setStyle("titlestyle", titleframe)
    key = scribus.createText(15.5, 43.5, 8, 18)
    scribus.setText("Name\n\nL\na\nb", key)
    SelectAllText(key)
    scribus.setStyle('keystyle', key)
    textlen = scribus.getTextLength(key)
    scribus.selectText(textlen-6, 6, key)
    scribus.setStyle('keyLabstyle', key)

    while (index < len(colors)):
        if ypos < 230:
            ypos = ypos + 35
        else:
            ypos = 30
            scribus.newPage(-1)
            currpage = currpage + 1
        for xpos in range(25, 180, 25):
           scribus.defineColorLab(colors[index], float(L[index]), float(a[index]), float(b[index]))
           newrectangle = scribus.createRect(xpos,ypos,22,12)
           scribus.setLineColor("None",newrectangle)
           scribus.setFillColor(colors[index], newrectangle)
           newtext = scribus.createText(xpos, ypos + 13.5, 22, 15)
           if (len(colors[index])<18):
               spacer = '\n'
           else:
               spacer = ''
           data = str(L[index]) + '\n' + str(a[index]) + '\n' + str(b[index])
           scribus.setText(colors[index]+spacer+'\n'+ data, newtext)
           SelectAllText(newtext)
           scribus.setStyle('keystyle',newtext)
           textlen = scribus.getTextLength(newtext)
           datalen = len(data)
           overflow = scribus.textOverflows(newtext)
           if overflow == 1:
               scribus.sizeObject(22,20, newtext)
           scribus.selectText(textlen - datalen, datalen, newtext)
           scribus.setStyle('keyLabstyle', newtext)
           index = index + 1
           if index >= len(colors):
               break
        scribus.progressSet(currpage)
pages = scribus.pageCount()
for p in range(1, pages + 1):
    scribus.applyMasterPage('attribution',p)
scribus.progressReset()
scribus.deselectAll()
#finish = datetime.datetime.now().replace(microsecond=0)
#interval = finish - start
#scribus.messageBox("Interval",'This took ' + str(interval) + ' to finish',button1=scribus.BUTTON_OK)
scribus.setRedraw(True)
scribus.redrawAll()
           
            
